package com.appdatatracker.views

import android.app.Service
import android.content.Context
import android.net.ConnectivityManager
import android.os.Build
import android.provider.Settings
import android.util.Log
import com.appdatatracker.cache.CacheConstants
import com.appdatatracker.cache.Prefs
import com.appdatatracker.model.BaseResponse
import com.appdatatracker.model.CheckData
import com.appdatatracker.networking.*
import com.appdatatracker.repository.NetworkOnlineDataRepo
import com.appdatatracker.utils.showAlertDialog
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.flowOn
import retrofit2.Response
import java.net.UnknownHostException

@ExperimentalCoroutinesApi
class AppTracker(private var context: Context) {
    private var listener: onTrackListener? = null

    var connectivityManager: ConnectivityManager? = null
    var apiService: ApiService? = null

    init {
        connectivityManager =
            context.getSystemService(Service.CONNECTIVITY_SERVICE) as ConnectivityManager
        apiService = provideApiService(provideApiProvider())

    }

    fun startTrack(
        applicationName: String,
        versionCode: String,
        string: String,
        trackCallback: onTrackListener
    ) {
        val data = addTrackData(applicationName, versionCode, string)
        listener = trackCallback
        if (isInternetActive()) {

            if (isAlreadyTracked()) {
                showDialog("TrackDownloadManager: Error- App download already tracked")

            } else {
                trackHandling(data)

            }

        } else {
            showDialog("TrackDownloadManager: Internet is offline ")
        }
    }

    private fun addTrackData(
        applicationName: String,
        versionCode: String,
        app_name: String
    ): CheckData {
        val data = CheckData().apply {
            bundleId = applicationName
            appName = app_name
            deviceId =
                Settings.Secure.getString(
                    context.contentResolver,
                    Settings.Secure.ANDROID_ID
                )
            modelNumber = Build.MODEL
            buildVersion = versionCode
        }
        Log.e("TRACK_DATA", data.toString())
        return data

    }

    private fun trackHandling(data: CheckData) {
        GlobalScope.launch {
            val response = sendDataToServer(data)
            withContext(Dispatchers.Main) {
                response.collect {
                    handleResponse(it)

                }
            }
        }
    }

    private fun isAlreadyTracked(): Boolean {
        return Prefs.getBoolean(CacheConstants.IS_TRACKED, false) == true
    }

    private fun isInternetActive(): Boolean {
        return connectivityManager?.activeNetworkInfo?.isConnected == true
    }

    private fun handleResponse(it: DataResult<BaseResponse>) {
        when (it) {
            is DataResult.Loading -> {
                Log.e("DATAAA", "loading")
            }
            is DataResult.Success -> {
                Prefs.save(CacheConstants.IS_TRACKED, true)
                showDialog(
                    "TrackDownloadManager: SUCCESS- App Download Tracked"
                )
            }
            is DataResult.Failure -> {
                if ((it.errorCode != null && it.errorCode == HTTPStatus.NETWORK_ISSUE) || (it.exception != null && it.exception is UnknownHostException)) {
                    showDialog("TrackDownloadManager: Internet is offline ")
                } else {
                    showDialog("TrackDownloadManager: Error- App download could not be tracked, ${it.message}  ")
                }


            }
        }
    }

    private fun sendDataToServer(data: CheckData): Flow<DataResult<BaseResponse>> {
        return object : NetworkOnlineDataRepo<BaseResponse, BaseResponse>() {
            override suspend fun fetchDataFromRemoteSource(): Response<BaseResponse> {
                return apiService!!.checkData(data)
            }
        }.asFlow().flowOn(Dispatchers.IO)
    }

    private fun showDialog(message: String) {
        listener?.onDataTracked(message)
    }

    interface onTrackListener {
        fun onDataTracked(message: String)
    }

}